/*
 * Copyright (C) 2013, 2014 Giorgio Vazzana
 *
 * This file is part of Seren.
 *
 * Seren is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Seren is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include "nc.h"
#include "common.h"
#include "msgbook.h"

#define MODULE "ncurses"

struct NContext nctx;

int nc_init(const char *version, const char *nick, int theme)
{
	initscr();
	if (has_colors() == FALSE) {
		endwin();
		msgbook_enqueue(&mb0, MB_TYPE_ERROR, MODULE, "Your terminal does not support color");
		return -1;
	}
	start_color();
	if (theme <=0 || theme > 2) { /* classic theme */
		/* main windows / messages colors */
		assume_default_colors(COLOR_WHITE, COLOR_BLUE);
		init_pair(1, COLOR_GREEN,  COLOR_BLUE);
		init_pair(2, COLOR_YELLOW, COLOR_WHITE);
		init_pair(3, COLOR_RED,    COLOR_WHITE);
		init_pair(4, COLOR_WHITE,  COLOR_BLUE);
		/* bars colors */
		init_pair(10, COLOR_BLACK,  COLOR_WHITE);
		init_pair(11, COLOR_GREEN,  COLOR_WHITE);
		init_pair(12, COLOR_YELLOW, COLOR_WHITE);
		init_pair(13, COLOR_RED,    COLOR_WHITE);
	} else if (theme == 1) { /* dark theme */
		assume_default_colors(COLOR_WHITE, COLOR_BLACK);
		init_pair(1, COLOR_GREEN,  COLOR_BLACK);
		init_pair(2, COLOR_YELLOW, COLOR_BLACK);
		init_pair(3, COLOR_RED,    COLOR_BLACK);
		init_pair(4, COLOR_CYAN,   COLOR_BLACK);

		init_pair(10, COLOR_BLACK,  COLOR_WHITE);
		init_pair(11, COLOR_GREEN,  COLOR_WHITE);
		init_pair(12, COLOR_YELLOW, COLOR_WHITE);
		init_pair(13, COLOR_RED,    COLOR_WHITE);
	} else if (theme == 2) { /* clear theme */
		assume_default_colors(COLOR_BLACK, COLOR_WHITE);
		init_pair(1, COLOR_GREEN,  COLOR_WHITE);
		init_pair(2, COLOR_YELLOW, COLOR_WHITE);
		init_pair(3, COLOR_RED,    COLOR_WHITE);
		init_pair(4, COLOR_CYAN,   COLOR_WHITE);

		init_pair(10, COLOR_WHITE,  COLOR_CYAN);
		init_pair(11, COLOR_WHITE,  COLOR_CYAN);
		init_pair(12, COLOR_YELLOW, COLOR_CYAN);
		init_pair(13, COLOR_RED,    COLOR_CYAN);
	}
	cbreak();
	noecho();
	keypad(stdscr, TRUE);
	getmaxyx(stdscr, nctx.rows, nctx.cols);

	/* create message window */
	nctx.win = newwin(nctx.rows - 4, nctx.cols, 2, 0);
	if (nctx.win == NULL) {
		endwin();
		snprintf(msgbuf, MBS, "Could not create ncurses window");
		msgbook_enqueue(&mb0, MB_TYPE_ERROR, MODULE, msgbuf);
		return -1;
	}
	scrollok(nctx.win, TRUE);

	print_bars(version);
	print_prompt(nick);

	nctx.initialized = 1;

	return 0;
}

void nc_close(void)
{
	/* shut down ncurses */
	if (nctx.initialized) {
		endwin();
		nctx.initialized = 0;
		mb0.print_cb = msgbook_print_cb_stderr;
		msgbook_flush(&mb0);
	}
}

void print_bars(const char *version)
{
	/* title bar */
	move(0, 0);
	printw(" Seren, ver. %s", version);
	move(0, 0);
	chgat(-1, A_NORMAL, 10, NULL);

	/* status bar */
	move(nctx.rows - 2, 0);
	addch(' ');
	move(nctx.rows - 2, 0);
	chgat(-1, A_NORMAL, 10, NULL);

	refresh();
}

void print_prompt(const char *nick)
{
	move(nctx.rows - 1, 0);
	printw("%s> ", nick);
	refresh();
}

void clear_input_line(int delete_buffer)
{
	if (delete_buffer) {
		nctx.linepos  = 0;
		nctx.wlinepos = 0;
	}
	move(nctx.rows - 1, 0);
	clrtoeol();
	refresh();
}

void nc_print_cb(time_t ts, unsigned int type, const char *module, const char *msg)
{
	int y, x;

	getyx(stdscr, y, x);
	if (type == MB_TYPE_DEBUG || type == MB_TYPE_VERBOSE) {
		wattron(nctx.win, COLOR_PAIR(1));
		wprintw(nctx.win, "[%s][%s] %s\n", ts2time(ts), module, msg);
		wattroff(nctx.win, COLOR_PAIR(1));
	} else if (type == MB_TYPE_WARNING) {
		wattron(nctx.win, COLOR_PAIR(2));
		wprintw(nctx.win, "[%s][%s] %s\n", ts2time(ts), module, msg);
		wattroff(nctx.win, COLOR_PAIR(2));
	} else if (type == MB_TYPE_ERROR) {
		wattron(nctx.win, COLOR_PAIR(3));
		wprintw(nctx.win, "[%s][%s] %s\n", ts2time(ts), module, msg);
		wattroff(nctx.win, COLOR_PAIR(3));
	} else if (type == MB_TYPE_GUI) {
		wattron(nctx.win, COLOR_PAIR(4));
		wprintw(nctx.win, "[%s][%s] %s\n", ts2time(ts), module, msg);
		wattroff(nctx.win, COLOR_PAIR(4));
	} else if (type == MB_TYPE_CHAT) {
		wattron(nctx.win, A_BOLD);
		wprintw(nctx.win, "[%s] %s> %s\n", ts2time(ts), module, msg);
		wattroff(nctx.win, A_BOLD);
	} else {
		wprintw(nctx.win, "[%s][%s] %s\n", ts2time(ts), module, msg);
	}
	wrefresh(nctx.win);
	move(y, x);
	refresh();
}

#undef MODULE
